package systems.kumi.usbdemo

import android.content.Intent
import android.net.Uri
import android.os.Build
import android.os.Bundle
import android.os.Environment
import android.os.StatFs
import android.provider.Settings
import android.util.Log
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.activity.result.contract.ActivityResultContracts
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.material3.Button
import androidx.compose.material3.ButtonDefaults
import androidx.compose.material3.Text
import androidx.compose.runtime.*
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.documentfile.provider.DocumentFile
import java.nio.file.Files.createFile

class MainActivity : ComponentActivity() {

    private val SAVED_URI = "savedUri"
    private var currentUri: Uri? = null
    private var directoryPath by mutableStateOf("")
    private var directoryContents by mutableStateOf(emptyList<String>())

    private val openFolder =
        registerForActivityResult(ActivityResultContracts.OpenDocumentTree()) { uri: Uri? ->
            if (uri != null) {
                contentResolver.takePersistableUriPermission(
                    uri,
                    Intent.FLAG_GRANT_READ_URI_PERMISSION or Intent.FLAG_GRANT_WRITE_URI_PERMISSION
                )
                getPreferences(MODE_PRIVATE).edit().putString(SAVED_URI, uri.toString()).apply()
                currentUri = uri
                handleUriSelection(uri)
            }
        }


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        val onClick: () -> Unit = {
            if (currentUri != null) {
                createFile(currentUri!!)
            } else {
                Log.e("Oops", "currentUri is null?")
            }
        }

        setContent {
            FileSystemContent(directoryPath, directoryContents, onClick)
        }

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
            if (!Environment.isExternalStorageManager()) {
                val intent = Intent(Settings.ACTION_MANAGE_ALL_FILES_ACCESS_PERMISSION)
                startActivity(intent)
            } else {
                processIntent()
            }
        } else {
            processIntent()
        }
    }

    override fun onResume() {
        super.onResume()
        processIntent()
    }

    private fun processIntent() {
        val uriString = getPreferences(MODE_PRIVATE).getString(SAVED_URI, null)

        if (uriString != null) {
            val uri = Uri.parse(uriString)
            currentUri = uri
            handleUriSelection(uri)
        } else {
            openFolder.launch(null)
        }
    }

    private fun handleUriSelection(uri: Uri) {
        directoryPath = "Chosen directory: ${uri.path}\n"

        val directory = DocumentFile.fromTreeUri(this, uri)
        directoryContents = directory?.listFiles()?.map {
            "${it.name ?: ""}, directory = ${it.isDirectory}, size = ${it.length()} bytes"
        } ?: emptyList()
    }

    private fun createFile(uri: Uri) {
        val documentsTree = DocumentFile.fromTreeUri(this, uri)
        val timestamp = System.currentTimeMillis().toString()
        documentsTree?.createFile("text/plain", timestamp)?.let {
            val outputStream = contentResolver.openOutputStream(it.uri)
            outputStream?.close()
            handleUriSelection(uri)
        }
    }
}

@Composable
fun FileSystemContent(directoryPath: String, directoryContents: List<String>, onButtonClick: () -> Unit) {
    Box(Modifier.fillMaxSize()) {
        Column {
            Button(onClick = onButtonClick, colors = ButtonDefaults.buttonColors(containerColor = Color(88, 208, 98))) {
                Text("Create File")
            }
            LazyColumn() {
                item {
                    Text(text = directoryPath, color = Color(88, 208, 98),
                        fontSize = 16.sp, modifier = Modifier.padding(all = 8.dp))
                }
                items(items = directoryContents) { fileName ->
                    Text(text = fileName, color = Color.Black, fontSize = 14.sp,
                        modifier = Modifier.padding(all = 4.dp))
                }
            }
        }
    }
}
